@@ -183,6 +183,18 @@ const char *get_atto(const xs_dict *msg)
 }
 
 
+const char *get_in_reply_to(const xs_dict *msg)
+/* gets the inReplyTo id */
+{
+    const xs_val *in_reply_to = xs_dict_get(msg, "inReplyTo");
+
+    if (xs_type(in_reply_to) == XSTYPE_DICT)
+        in_reply_to = xs_dict_get(in_reply_to, "id");
+
+    return in_reply_to;
+}
+
+
 xs_list *get_attachments(const xs_dict *msg)
 /* unify the garbage fire that are the attachments */
 {
@@ -202,7 +214,7 @@ xs_list *get_attachments(const xs_dict *msg)
         else
             attach = xs_dup(p);
 
-        if (xs_type(attach) == XSTYPE_LIST) {
+        if (xs_type(attach) == XSTYPE_LIST && xs_list_len(attach) == 0) {
             /* does the message have an image? */
             const xs_dict *d = xs_dict_get(msg, "image");
             if (xs_type(d) == XSTYPE_DICT) {
@@ -373,11 +385,14 @@ int timeline_request(snac *snac, const char **id, xs_str **wrk, int level)
                             }
 
                             /* does it have an ancestor? */
-                            const char *in_reply_to = xs_dict_get(object, "inReplyTo");
+                            const char *in_reply_to = get_in_reply_to(object);
 
                             /* store */
                             timeline_add(snac, nid, object);
 
+                            /* redistribute to lists for this user */
+                            list_distribute(snac, actor, object);
+
                             /* recurse! */
                             timeline_request(snac, &in_reply_to, NULL, level + 1);
                         }
@@ -419,7 +434,7 @@ int send_to_inbox(snac *snac, const xs_str *inbox, const xs_dict *msg,
 }
 
 
-xs_str *get_actor_inbox(const char *actor)
+xs_str *get_actor_inbox(const char *actor, int shared)
 /* gets an actor's inbox */
 {
     xs *data = NULL;
@@ -427,7 +442,7 @@ xs_str *get_actor_inbox(const char *actor)
 
     if (valid_status(actor_request(NULL, actor, &data))) {
         /* try first endpoints/sharedInbox */
-        if ((v = xs_dict_get(data, "endpoints")))
+        if (shared && (v = xs_dict_get(data, "endpoints")))
             v = xs_dict_get(v, "sharedInbox");
 
         /* try then the regular inbox */
@@ -444,7 +459,7 @@ int send_to_actor(snac *snac, const char *actor, const xs_dict *msg,
 /* sends a message to an actor */
 {
     int status = HTTP_STATUS_BAD_REQUEST;
-    xs *inbox = get_actor_inbox(actor);
+    xs *inbox = get_actor_inbox(actor, 1);
 
     if (!xs_is_null(inbox))
         status = send_to_inbox(snac, inbox, msg, payload, p_size, timeout);
@@ -668,7 +683,7 @@ int is_msg_for_me(snac *snac, const xs_dict *c_msg)
         return 3;
 
     /* is this message a reply to another? */
-    const char *irt = xs_dict_get(msg, "inReplyTo");
+    const char *irt = get_in_reply_to(msg);
     if (!xs_is_null(irt)) {
         xs *r_msg = NULL;
 
@@ -721,7 +736,7 @@ xs_str *process_tags(snac *snac, const char *content, xs_list **tag)
         /* use this same server */
         def_srv = xs_dup(xs_dict_get(srv_config, "host"));
 
-    split = xs_regex_split(content, "(@[A-Za-z0-9_]+(@[A-Za-z0-9\\.-]+)?|&#[0-9]+;|#[^ ,\\.:;<]+)");
+    split = xs_regex_split(content, "(@[A-Za-z0-9_]+(@[A-Za-z0-9\\.-]+)?|&#[0-9]+;|#(_|[^[:punct:][:space:]])+)");
 
     p = split;
     while (xs_list_iter(&p, &v)) {
@@ -757,7 +772,7 @@ xs_str *process_tags(snac *snac, const char *content, xs_list **tag)
 
                     tl = xs_list_append(tl, d);
 
-                    link = xs_fmt("<a href=\"%s\" class=\"u-url h-card mention\">%s</a>", actor, n);
+                    link = xs_fmt("<span class=\"h-card\"><a href=\"%s\" class=\"u-url mention\">%s</a></span>", actor, n);
                 }
 
                 if (!xs_is_null(link))
@@ -1158,12 +1173,12 @@ xs_dict *msg_actor(snac *snac)
     msg = xs_dict_set(msg, "preferredUsername", snac->uid);
     msg = xs_dict_set(msg, "published",         xs_dict_get(snac->config, "published"));
 
-    xs *f_bio_2 = not_really_markdown(xs_dict_get(snac->config, "bio"), NULL, NULL);
+    xs *f_bio_2 = not_really_markdown(xs_dict_get(snac->config, "bio"), NULL, &tags);
     f_bio = process_tags(snac, f_bio_2, &tags);
     msg = xs_dict_set(msg, "summary", f_bio);
     msg = xs_dict_set(msg, "tag", tags);
 
-    char *folders[] = { "inbox", "outbox", "followers", "following", NULL };
+    char *folders[] = { "inbox", "outbox", "followers", "following", "featured", NULL };
     for (n = 0; folders[n]; n++) {
         xs *f = xs_fmt("%s/%s", snac->actor, folders[n]);
         msg = xs_dict_set(msg, folders[n], f);
@@ -1203,7 +1218,30 @@ xs_dict *msg_actor(snac *snac)
     }
 
     /* add the metadata as attachments of PropertyValue */
-    const xs_dict *metadata = xs_dict_get(snac->config, "metadata");
+    xs *metadata = NULL;
+    const xs_dict *md = xs_dict_get(snac->config, "metadata");
+
+    if (xs_type(md) == XSTYPE_DICT)
+        metadata = xs_dup(md);
+    else
+    if (xs_type(md) == XSTYPE_STRING) {
+        metadata = xs_dict_new();
+        xs *l = xs_split(md, "\n");
+        const char *ll;
+
+        xs_list_foreach(l, ll) {
+            xs *kv = xs_split_n(ll, "=", 1);
+            const char *k = xs_list_get(kv, 0);
+            const char *v = xs_list_get(kv, 1);
+
+            if (k && v) {
+                xs *kk = xs_strip_i(xs_dup(k));
+                xs *vv = xs_strip_i(xs_dup(v));
+                metadata = xs_dict_set(metadata, kk, vv);
+            }
+        }
+    }
+
     if (xs_type(metadata) == XSTYPE_DICT) {
         xs *attach = xs_list_new();
         const xs_str *k;
@@ -1241,6 +1279,18 @@ xs_dict *msg_actor(snac *snac)
         msg = xs_dict_set(msg, "endpoints", d);
     }
 
+    /* does this user have an aka? */
+    const char *aka = xs_dict_get(snac->config, "alias");
+    if (xs_type(aka) == XSTYPE_STRING && *aka) {
+        xs *loaka = xs_list_append(xs_list_new(), aka);
+
+        msg = xs_dict_set(msg, "alsoKnownAs", loaka);
+    }
+
+    const xs_val *manually = xs_dict_get(snac->config, "approve_followers");
+    msg = xs_dict_set(msg, "manuallyApprovesFollowers",
+        xs_stock(xs_is_true(manually) ? XSTYPE_TRUE : XSTYPE_FALSE));
+
     return msg;
 }
 
@@ -1514,6 +1564,17 @@ xs_dict *msg_pong(snac *user, const char *rcpt, const char *object)
 }
 
 
+xs_dict *msg_move(snac *user, const char *new_account)
+/* creates a Move message (to move the user to new_account) */
+{
+    xs_dict *msg = msg_base(user, "Move", "@dummy", user->actor, NULL, user->actor);
+
+    msg = xs_dict_append(msg, "target", new_account);
+
+    return msg;
+}
+
+
 xs_dict *msg_question(snac *user, const char *content, xs_list *attach,
                       const xs_list *opts, int multiple, int end_secs)
 /* creates a Question message */
@@ -1721,7 +1782,7 @@ int process_input_message(snac *snac, const xs_dict *msg, const xs_dict *req)
         type = "Note";
 
     /* reject uninteresting messages right now */
-    if (xs_match(type, "Add|View")) {
+    if (xs_match(type, "Add|View|Reject|Read|Remove")) {
         srv_debug(0, xs_fmt("Ignored message of type '%s'", type));
         return -1;
     }
@@ -1753,7 +1814,7 @@ int process_input_message(snac *snac, const xs_dict *msg, const xs_dict *req)
         if (xs_type(obj_id) == XSTYPE_DICT)
             obj_id = xs_dict_get(obj_id, "id");
 
-        if (!object_here(obj_id)) {
+        if (xs_is_null(obj_id) || !object_here(obj_id)) {
             srv_debug(1, xs_fmt("dropped 'Delete' message from unknown object '%s'", obj_id));
             return -1;
         }
@@ -1852,6 +1913,12 @@ int process_input_message(snac *snac, const xs_dict *msg, const xs_dict *req)
     }
 
     if (strcmp(type, "Follow") == 0) { /** **/
+        const char *id = xs_dict_get(msg, "id");
+
+        if (xs_is_null(id)) {
+            snac_log(snac, xs_fmt("malformed message: no 'id' field"));
+        }
+        else
         if (!follower_check(snac, actor)) {
             /* ensure the actor object is here */
             if (!object_here(actor)) {
@@ -1860,22 +1927,31 @@ int process_input_message(snac *snac, const xs_dict *msg, const xs_dict *req)
                 object_add(actor, actor_obj);
             }
 
-            xs *f_msg = xs_dup(msg);
-            xs *reply = msg_accept(snac, f_msg, actor);
+            if (xs_is_true(xs_dict_get(snac->config, "approve_followers"))) {
+                pending_add(snac, actor, msg);
 
-            post_message(snac, actor, reply);
+                snac_log(snac, xs_fmt("new pending follower approval %s", actor));
+            }
+            else {
+                /* automatic following */
+                xs *f_msg = xs_dup(msg);
+                xs *reply = msg_accept(snac, f_msg, actor);
 
-            if (xs_is_null(xs_dict_get(f_msg, "published"))) {
-                /* add a date if it doesn't include one (Mastodon) */
-                xs *date = xs_str_utctime(0, ISO_DATE_SPEC);
-                f_msg = xs_dict_set(f_msg, "published", date);
+                post_message(snac, actor, reply);
+
+                if (xs_is_null(xs_dict_get(f_msg, "published"))) {
+                    /* add a date if it doesn't include one (Mastodon) */
+                    xs *date = xs_str_utctime(0, ISO_DATE_SPEC);
+                    f_msg = xs_dict_set(f_msg, "published", date);
+                }
+
+                timeline_add(snac, id, f_msg);
+
+                follower_add(snac, actor);
+
+                snac_log(snac, xs_fmt("new follower %s", actor));
             }
 
-            timeline_add(snac, xs_dict_get(f_msg, "id"), f_msg);
-
-            follower_add(snac, actor);
-
-            snac_log(snac, xs_fmt("new follower %s", actor));
             do_notify = 1;
         }
         else
@@ -1896,6 +1972,11 @@ int process_input_message(snac *snac, const xs_dict *msg, const xs_dict *req)
                     snac_log(snac, xs_fmt("no longer following us %s", actor));
                     do_notify = 1;
                 }
+                else
+                if (pending_check(snac, actor)) {
+                    pending_del(snac, actor);
+                    snac_log(snac, xs_fmt("cancelled pending follow from %s", actor));
+                }
                 else
                     snac_log(snac, xs_fmt("error deleting follower %s", actor));
             }
@@ -1929,7 +2010,7 @@ int process_input_message(snac *snac, const xs_dict *msg, const xs_dict *req)
 
         if (xs_match(utype, "Note|Article")) { /** **/
             const char *id          = xs_dict_get(object, "id");
-            const char *in_reply_to = xs_dict_get(object, "inReplyTo");
+            const char *in_reply_to = get_in_reply_to(object);
             const char *atto        = get_atto(object);
             xs *wrk           = NULL;
 
@@ -1969,6 +2050,9 @@ int process_input_message(snac *snac, const xs_dict *msg, const xs_dict *req)
         if (strcmp(utype, "Question") == 0) { /**  **/
             const char *id = xs_dict_get(object, "id");
 
+            if (xs_is_null(id))
+                snac_log(snac, xs_fmt("malformed message: no 'id' field"));
+            else
             if (timeline_add(snac, id, object))
                 snac_log(snac, xs_fmt("new 'Question' %s %s", actor, id));
         }
@@ -1976,6 +2060,9 @@ int process_input_message(snac *snac, const xs_dict *msg, const xs_dict *req)
         if (strcmp(utype, "Video") == 0) { /** **/
             const char *id = xs_dict_get(object, "id");
 
+            if (xs_is_null(id))
+                snac_log(snac, xs_fmt("malformed message: no 'id' field"));
+            else
             if (timeline_add(snac, id, object))
                 snac_log(snac, xs_fmt("new 'Video' %s %s", actor, id));
         }
@@ -2018,10 +2105,13 @@ int process_input_message(snac *snac, const xs_dict *msg, const xs_dict *req)
         if (xs_type(object) == XSTYPE_DICT)
             object = xs_dict_get(object, "id");
 
+        if (xs_is_null(object))
+            snac_log(snac, xs_fmt("malformed message: no 'id' field"));
+        else
         if (timeline_admire(snac, object, actor, 1) == HTTP_STATUS_CREATED)
             snac_log(snac, xs_fmt("new '%s' %s %s", type, actor, object));
         else
-            snac_log(snac, xs_fmt("repeated 'Like' from %s to %s", actor, object));
+            snac_log(snac, xs_fmt("repeated '%s' from %s to %s", type, actor, object));
 
         do_notify = 1;
     }
@@ -2030,6 +2120,9 @@ int process_input_message(snac *snac, const xs_dict *msg, const xs_dict *req)
         if (xs_type(object) == XSTYPE_DICT)
             object = xs_dict_get(object, "id");
 
+        if (xs_is_null(object))
+            snac_log(snac, xs_fmt("malformed message: no 'id' field"));
+        else
         if (is_muted(snac, actor) && !xs_startswith(object, snac->actor))
             snac_log(snac, xs_fmt("dropped 'Announce' from muted actor %s", actor));
         else
@@ -2082,6 +2175,9 @@ int process_input_message(snac *snac, const xs_dict *msg, const xs_dict *req)
         if (xs_match(utype, "Note|Page|Article|Video")) { /** **/
             const char *id = xs_dict_get(object, "id");
 
+            if (xs_is_null(id))
+                snac_log(snac, xs_fmt("malformed message: no 'id' field"));
+            else
             if (object_here(id)) {
                 object_add_ow(id, object);
                 timeline_touch(snac);
@@ -2096,13 +2192,17 @@ int process_input_message(snac *snac, const xs_dict *msg, const xs_dict *req)
             const char *id     = xs_dict_get(object, "id");
             const char *closed = xs_dict_get(object, "closed");
 
-            object_add_ow(id, object);
-            timeline_touch(snac);
+            if (xs_is_null(id))
+                snac_log(snac, xs_fmt("malformed message: no 'id' field"));
+            else {
+                object_add_ow(id, object);
+                timeline_touch(snac);
 
-            snac_log(snac, xs_fmt("%s poll %s", closed == NULL ? "updated" : "closed", id));
+                snac_log(snac, xs_fmt("%s poll %s", closed == NULL ? "updated" : "closed", id));
 
-            if (closed != NULL)
-                do_notify = 1;
+                if (closed != NULL)
+                    do_notify = 1;
+            }
         }
         else {
             srv_archive_error("unsupported_update", "unsupported_update", req, msg);
@@ -2115,6 +2215,9 @@ int process_input_message(snac *snac, const xs_dict *msg, const xs_dict *req)
         if (xs_type(object) == XSTYPE_DICT)
             object = xs_dict_get(object, "id");
 
+        if (xs_is_null(object))
+            snac_log(snac, xs_fmt("malformed message: no 'id' field"));
+        else
         if (object_here(object)) {
             timeline_del(snac, object);
             snac_debug(snac, 1, xs_fmt("new 'Delete' %s %s", actor, object));
@@ -2128,20 +2231,25 @@ int process_input_message(snac *snac, const xs_dict *msg, const xs_dict *req)
     }
     else
     if (strcmp(type, "Ping") == 0) { /** **/
+        const char *id = xs_dict_get(msg, "id");
+
         snac_log(snac, xs_fmt("'Ping' requested from %s", actor));
 
-        xs *rsp = msg_pong(snac, actor, xs_dict_get(msg, "id"));
+        if (!xs_is_null(id)) {
+            xs *rsp = msg_pong(snac, actor, id);
 
-        enqueue_output_by_actor(snac, rsp, actor, 0);
+            enqueue_output_by_actor(snac, rsp, actor, 0);
+        }
     }
     else
     if (strcmp(type, "Block") == 0) { /** **/
-        snac_log(snac, xs_fmt("'Block' received from %s", actor));
+        snac_debug(snac, 1, xs_fmt("'Block' received from %s", actor));
 
         /* should we MUTE the actor back? */
         /* mute(snac, actor); */
 
-        do_notify = 1;
+        if (!xs_is_true(xs_dict_get(srv_config, "disable_block_notifications")))
+            do_notify = 1;
     }
     else
     if (strcmp(type, "Move") == 0) { /** **/
@@ -2190,8 +2298,12 @@ int process_input_message(snac *snac, const xs_dict *msg, const xs_dict *req)
                     else
                         snac_log(snac, xs_fmt("'Move' error: cannot get %s 'alsoKnownAs'", new_account));
                 }
-                else
+                else {
                     snac_log(snac, xs_fmt("'Move' error: cannot get new actor %s", new_account));
+
+                    /* may be a server hiccup, retry later */
+                    return 0;
+                }
             }
             else
                 snac_log(snac, xs_fmt("'Move' error: actor %s is not being followed", old_account));
@@ -2267,7 +2379,7 @@ void process_user_queue_item(snac *snac, xs_dict *q_item)
         /* iterate the recipients */
         c = 0;
         while (xs_list_next(rcpts, &actor, &c)) {
-            xs *inbox = get_actor_inbox(actor);
+            xs *inbox = get_actor_inbox(actor, 1);
 
             if (inbox != NULL) {
                 /* add to the set and, if it's not there, send message */
@@ -2305,8 +2417,6 @@ void process_user_queue_item(snac *snac, xs_dict *q_item)
             return;
 
         if (!process_input_message(snac, msg, req)) {
-            srv_archive_error("input", "process_input_message", req, msg);
-
             if (retries > queue_retry_max)
                 snac_log(snac, xs_fmt("input giving up"));
             else {
@@ -2612,7 +2722,6 @@ void process_queue_item(xs_dict *q_item)
             unlink(tmpfn);
 
             if (cnt == 0) {
-                srv_archive_qitem("no_valid_recipients", q_item);
                 srv_debug(1, xs_fmt("no valid recipients for %s", xs_dict_get(msg, "id")));
             }
         }
@@ -2644,6 +2753,58 @@ int process_queue(void)
 }
 
 
+/** account migration **/
+
+int migrate_account(snac *user)
+/* migrates this account to a new one (stored in the 'alias' user field) */
+{
+    const char *new_account = xs_dict_get(user->config, "alias");
+
+    if (xs_type(new_account) != XSTYPE_STRING) {
+        snac_log(user, xs_fmt("Cannot migrate: alias (destination account) not set"));
+        return 1;
+    }
+
+    xs *new_actor = NULL;
+    int status;
+
+    if (!valid_status(status = activitypub_request(user, new_account, &new_actor))) {
+        snac_log(user, xs_fmt("Cannot migrate: error requesting actor %s %d", new_account, status));
+        return 1;
+    }
+
+    actor_add(new_account, new_actor);
+
+    const char *loaka = xs_dict_get(new_actor, "alsoKnownAs");
+
+    if (xs_type(loaka) != XSTYPE_LIST) {
+        snac_log(user, xs_fmt("Cannot migrate: destination account doesn't have any aliases"));
+        return 1;
+    }
+
+    if (xs_list_in(loaka, user->actor) == -1) {
+        snac_log(user, xs_fmt("Cannot migrate: destination account doesn't have this one as an alias"));
+        return 1;
+    }
+
+    xs *move = msg_move(user, new_account);
+    xs *fwers = follower_list(user);
+
+    const char *actor;
+    xs_list_foreach(fwers, actor) {
+        /* get the actor inbox, excluding the shared one */
+        xs *inbox = get_actor_inbox(actor, 0);
+
+        if (xs_is_null(inbox))
+            snac_log(user, xs_fmt("migrate_account: cannot get inbox for actor %s", actor));
+        else
+            enqueue_output(user, move, inbox, 0, 0);
+    }
+
+    return 0;
+}
+
+
 /** HTTP handlers */
 
 int activitypub_get_handler(const xs_dict *req, const char *q_path,
@@ -2686,16 +2847,17 @@ int activitypub_get_handler(const xs_dict *req, const char *q_path,
         snac_debug(&snac, 0, xs_fmt("serving actor [%s]", ua ? ua : "No UA"));
     }
     else
-    if (strcmp(p_path, "outbox") == 0) {
-        xs *id = xs_fmt("%s/outbox", snac.actor);
-        xs *elems = timeline_simple_list(&snac, "public", 0, 20);
+    if (strcmp(p_path, "outbox") == 0 || strcmp(p_path, "featured") == 0) {
+        xs *id = xs_fmt("%s/%s", snac.actor, p_path);
         xs *list = xs_list_new();
         msg = msg_collection(&snac, id);
-        char *p;
         const char *v;
+        int tc = 0;
 
-        p = elems;
-        while (xs_list_iter(&p, &v)) {
+        /* get the public outbox or the pinned list */
+        xs *elems = *p_path == 'o' ? timeline_simple_list(&snac, "public", 0, 20) : pinned_list(&snac);
+
+        while (xs_list_next(elems, &v, &tc)) {
             xs *i = NULL;
 
             if (valid_status(object_get_by_md5(v, &i))) {
