@@ -1,7 +1,7 @@
 /* snac - A simple, minimalistic ActivityPub instance */
 /* copyright (c) 2022 - 2024 grunfink et al. / MIT license */
 
-#define VERSION "2.59"
+#define VERSION "2.65"
 
 #define USER_AGENT "snac/" VERSION
 
@@ -26,6 +26,7 @@ extern double disk_layout;
 extern xs_str *srv_basedir;
 extern xs_dict *srv_config;
 extern xs_str *srv_baseurl;
+extern xs_str *srv_proxy_token_seed;
 
 extern int dbglevel;
 
@@ -140,6 +141,12 @@ int follower_del(snac *snac, const char *actor);
 int follower_check(snac *snac, const char *actor);
 xs_list *follower_list(snac *snac);
 
+int pending_add(snac *user, const char *actor, const xs_dict *msg);
+int pending_check(snac *user, const char *actor);
+xs_dict *pending_get(snac *user, const char *actor);
+void pending_del(snac *user, const char *actor);
+xs_list *pending_list(snac *user);
+
 double timeline_mtime(snac *snac);
 int timeline_touch(snac *snac);
 int timeline_here(snac *snac, const char *md5);
@@ -165,6 +172,7 @@ xs_list *following_list(snac *snac);
 void mute(snac *snac, const char *actor);
 void unmute(snac *snac, const char *actor);
 int is_muted(snac *snac, const char *actor);
+xs_list *muted_list(snac *user);
 
 int is_bookmarked(snac *user, const char *id);
 int bookmark(snac *user, const char *id);
@@ -283,16 +291,18 @@ int check_signature(const xs_dict *req, xs_str **err);
 srv_state *srv_state_op(xs_str **fname, int op);
 void httpd(void);
 
-int webfinger_request_signed(snac *snac, const char *qs, char **actor, char **user);
-int webfinger_request(const char *qs, char **actor, char **user);
-int webfinger_get_handler(xs_dict *req, char *q_path,
-                          char **body, int *b_size, char **ctype);
+int webfinger_request_signed(snac *snac, const char *qs, xs_str **actor, xs_str **user);
+int webfinger_request(const char *qs, xs_str **actor, xs_str **user);
+int webfinger_request_fake(const char *qs, xs_str **actor, xs_str **user);
+int webfinger_get_handler(xs_dict *req, const char *q_path,
+                          xs_val **body, int *b_size, char **ctype);
 
 const char *default_avatar_base64(void);
 
 xs_str *process_tags(snac *snac, const char *content, xs_list **tag);
 
 const char *get_atto(const xs_dict *msg);
+const char *get_in_reply_to(const xs_dict *msg);
 xs_list *get_attachments(const xs_dict *msg);
 
 xs_dict *msg_admiration(snac *snac, const char *object, const char *type);
@@ -309,6 +319,8 @@ xs_dict *msg_actor(snac *snac);
 xs_dict *msg_update(snac *snac, const xs_dict *object);
 xs_dict *msg_ping(snac *user, const char *rcpt);
 xs_dict *msg_pong(snac *user, const char *rcpt, const char *object);
+xs_dict *msg_move(snac *user, const char *new_account);
+xs_dict *msg_accept(snac *snac, const xs_val *object, const char *to);
 xs_dict *msg_question(snac *user, const char *content, xs_list *attach,
                       const xs_list *opts, int multiple, int end_secs);
 
@@ -319,7 +331,7 @@ int send_to_inbox_raw(const char *keyid, const char *seckey,
                   xs_val **payload, int *p_size, int timeout);
 int send_to_inbox(snac *snac, const xs_str *inbox, const xs_dict *msg,
                   xs_val **payload, int *p_size, int timeout);
-xs_str *get_actor_inbox(const char *actor);
+xs_str *get_actor_inbox(const char *actor, int shared);
 int send_to_actor(snac *snac, const char *actor, const xs_dict *msg,
                   xs_val **payload, int *p_size, int timeout);
 int is_msg_public(const xs_dict *msg);
@@ -343,14 +355,17 @@ xs_str *encode_html(const char *str);
 
 xs_str *html_timeline(snac *user, const xs_list *list, int read_only,
                       int skip, int show, int show_more,
-                      char *title, char *page, int utl);
+                      const char *title, const char *page, int utl, const char *error);
 
 int html_get_handler(const xs_dict *req, const char *q_path,
-                     char **body, int *b_size, char **ctype, xs_str **etag);
+                     char **body, int *b_size, char **ctype,
+                     xs_str **etag, xs_str **last_modified);
+
 int html_post_handler(const xs_dict *req, const char *q_path,
                       char *payload, int p_size,
                       char **body, int *b_size, char **ctype);
-xs_str *timeline_to_rss(snac *user, const xs_list *timeline, char *title, char *link, char *desc);
+xs_str *timeline_to_rss(snac *user, const xs_list *timeline,
+                        const char *title, const char *link, const char *desc);
 
 int write_default_css(void);
 int snac_init(const char *_basedir);
@@ -387,6 +402,13 @@ void mastoapi_purge(void);
 
 void verify_links(snac *user);
 
+void export_csv(snac *user);
+int migrate_account(snac *user);
+
+void import_blocked_accounts_csv(snac *user, const char *fn);
+void import_following_accounts_csv(snac *user, const char *fn);
+void import_list_csv(snac *user, const char *fn);
+void import_csv(snac *user);
 
 typedef enum {
 #define HTTP_STATUS(code, name, text) HTTP_STATUS_ ## name = code,
@@ -401,3 +423,5 @@ typedef struct {
     char   *text;
 } t_announcement;
 t_announcement *announcement(double after);
+
+xs_str *make_url(const char *href, const char *proxy, int by_token);
